// 
// Copyright (C) 2016, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

// This namespace holds indicators in this folder and is required. Do not change it.
namespace NinjaTrader.NinjaScript.Indicators
{
	/// <summary>
	/// VolatilityStop Bands code by Andres Cifuentes
	/// The Volatility Stop Indicator helps define the current trend. This indicator plots 
	/// a red line above the price bars when a downward trend is detected, and a blue line below
	/// the bars when an upward trend is detected. These lines are commonly used as trailing stops,
	/// and this indicator is commonly used as an exit tool rather than an entry technique. When price crosses
	/// the VSTOP value, the trend reverses and VSTOP moves to the other side of the price.
	/// </summary>
	public class VolatilityStop : Indicator
	{
		private SMA		sma;
		private StdDev	stdDev;
		private ATR		atr;
		bool tend = true;

		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
			
				Name						= "VolatilityStop";
				IsOverlay					= true;
				IsSuspendedWhileInactive	= true;
				NumATR					    = 2;
				Period						= 14;

				AddPlot(new Stroke (Brushes.Red,2), PlotStyle.Dot, NinjaTrader.Custom.Resource.BollingerUpperBand);
				AddPlot(new Stroke (Brushes.Green,2), PlotStyle.Dot, NinjaTrader.Custom.Resource.BollingerLowerBand);
				
				
			}
			else if (State == State.Configure)
			{
				
				stdDev	= StdDev(Period);
				atr = ATR(Period);
			}
		}

		protected override void OnBarUpdate()
		{
			double close0	= Close[0];
			double atr0	= atr[0];
			
			if (CurrentBar == Period){
				Lower[0]		= close0 - NumATR * atr0;
					} 
			
			
			if (CurrentBar > Period){	
										
			if (tend==false && close0 > Upper[1]){
				tend=true;
				Lower[0]		= close0 - NumATR * atr0;
				return;
			}
		
			if (tend == true && close0<Lower[1]){
				tend=false;
				Upper[0]		= close0 + NumATR * atr0;
				return;
				}
		
			if (tend==true){
				Lower[0]		= Math.Max(Math.Max( Lower[1],close0 - NumATR * atr0),Lower[1]+(NumATR *(atr[1]-atr0)));
							}
			
			else {
				Upper[0]		=  Math.Min(Math.Min(Upper[1], close0 + NumATR * atr0),Upper[1]-(NumATR *(atr[1]-atr0)));
								}
				}
		}

		#region Properties
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> Lower
		{
			get { return Values[1]; }
		}
		
	

		[Range(0, int.MaxValue), NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name = "NumATR", GroupName = "NinjaScriptParameters", Order = 0)]
		public double NumATR
		{ get; set; }

		[Range(1, int.MaxValue), NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Period", GroupName = "NinjaScriptParameters", Order = 1)]
		public int Period
		{ get; set; }

		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> Upper
		{
			get { return Values[0]; }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private VolatilityStop[] cacheVolatilityStop;
		public VolatilityStop VolatilityStop(double numATR, int period)
		{
			return VolatilityStop(Input, numATR, period);
		}

		public VolatilityStop VolatilityStop(ISeries<double> input, double numATR, int period)
		{
			if (cacheVolatilityStop != null)
				for (int idx = 0; idx < cacheVolatilityStop.Length; idx++)
					if (cacheVolatilityStop[idx] != null && cacheVolatilityStop[idx].NumATR == numATR && cacheVolatilityStop[idx].Period == period && cacheVolatilityStop[idx].EqualsInput(input))
						return cacheVolatilityStop[idx];
			return CacheIndicator<VolatilityStop>(new VolatilityStop(){ NumATR = numATR, Period = period }, input, ref cacheVolatilityStop);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.VolatilityStop VolatilityStop(double numATR, int period)
		{
			return indicator.VolatilityStop(Input, numATR, period);
		}

		public Indicators.VolatilityStop VolatilityStop(ISeries<double> input , double numATR, int period)
		{
			return indicator.VolatilityStop(input, numATR, period);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.VolatilityStop VolatilityStop(double numATR, int period)
		{
			return indicator.VolatilityStop(Input, numATR, period);
		}

		public Indicators.VolatilityStop VolatilityStop(ISeries<double> input , double numATR, int period)
		{
			return indicator.VolatilityStop(input, numATR, period);
		}
	}
}

#endregion
